/*! \file
    \brief Debugging message file

    This file provides mappings from various code numbers to strings.\n
    It is only used for debugging messages.
*/
#include <stdlib.h>
#include "CmdTable.h"

//! Command String from Command #
static const char *const commandTable[]= {
"CMD_OK",                // 0  deprecated
NULL,                    // 1
"CMD_FAILED",            // 2  command execution failed (incorrect parameters, etc)
"CMD_UNKNOWN",           // 3  unknown command
NULL,                    // 4
NULL,                    // 5
NULL,                    // 6
NULL,                    // 7
"CMD_DEBUG",             // 8  Debugging commands (parameter determines actual command)
NULL,                    // 9
"CMD_OPTION",            // 10 Get/set various options
NULL,                    // 11
"CMD_GET_VER",           // 12 returns 16 bit HW/SW version number, (major & minor revision
"CMD_GET_LAST_STATUS",   // 13 returns status of the previous command
"CMD_SET_BOOT",          // 14 request boot-loader firmware upgrade on next power-up, parameters: 'B','O','O','T', returns: none
NULL,                    // 15
NULL,                    // 16
NULL,                    // 17
NULL,                    // 18
NULL,                    // 19
NULL,                    // 20
NULL,                    // 21
NULL,                    // 22
NULL,                    // 23
NULL,                    // 24
NULL,                    // 25
NULL,                    // 26
NULL,                    // 27
NULL,                    // 28
NULL,                    // 29
"CMD_SET_TARGET",        // 30 set target, 8bit parameter: 00=HC12/HCS12(default), 01=HCS08
"CMD_CONNECT",           // 31 try to connect to the target
NULL,                    // 32
"CMD_RESET",             // 33 8bit parameter: 0=reset to Special Mode, 1=reset to Normal mode
"CMD_GET_STATUS",        // 34 returns 16bit status word:
"CMD_READ_BD",           // 35 parameter: 16-bit address, returns 8-bit value read from address;
"CMD_WRITE_BD",          // 36 parameter: 16-bit address, 8-bit value to write;
NULL,                    // 37
NULL,                    // 38
"CMD_HALT",              // 39 stop the CPU and bring it into background mode
"CMD_SET_SPEED",         // 40 deprecated
"CMD_READ_SPEED1",       // 41 read speed of the target: returns 16-bit tick count
"CMD_GO1",               // 42 start code execution
NULL,                    // 43
"CMD_STEP1",             // 44 perform single step
NULL,                    // 45
"CMD_SET_SPEED1",        // 46 sets-up the BDM interface for a new bit rate & tries
"CMD_CF_SET_DERIVATIVE", // 47
NULL,                    // 48
NULL,                    // 49
"CMD_READ_8",            // 50 parameter 16bit address, returns 8bit value read from address
"CMD_READ_16",           // 51 parameter 16bit address, returns 16bit value read from address
NULL,                    // 52
"CMD_READ_REGS",         // 53 reads registers, returns 16bit values:
NULL,                    // 54
"CMD_READ_BLOCK1",       // 55 parameter 16bit address, 8bit count of bytes to read,
NULL,                    // 56
NULL,                    // 57
NULL,                    // 58
NULL,                    // 59
"CMD_WRITE_8",           // 60 parameter 16bit address, 8bit value to write
"CMD_WRITE_16",          // 61 parameter 16bit address, 16bit value to write
NULL,                    // 62
NULL,                    // 63
NULL,                    // 64
"CMD_WRITE_BLOCK1",      // 65 parameters: 16bit address, 8bit count of bytes,
NULL,                    // 66
NULL,                    // 67
NULL,                    // 68
NULL,                    // 69
NULL,                    // 70
NULL,                    // 71
NULL,                    // 72
NULL,                    // 73
NULL,                    // 74
NULL,                    // 75
NULL,                    // 76
NULL,                    // 77
NULL,                    // 78
NULL,                    // 79
"CMD_WRITE_REG_PC",      // 80 parameter: 16-bit PC value
"CMD_WRITE_REG_SP",      // 81 parameter: 16-bit SP value
"CMD_WRITE_REG_X",       // 82 parameter: 16-bit IX (H:X) value
"CMD_WRITE_REG_Y",       // 83 parameter: 16-bit IY value
"CMD_WRITE_REG_D",       // 84 parameter: 16-bit B:A (x:A) value
"CMD_WRITE_REG_CCR",     // 85 parameter: 16-bit CCR (x:CCR) value
"CMD_CF_READ_CSR2",      // 86
"CMD_CF_READ_CSR3",      // 87
"CMD_CF_WRITE_CSR2",     // 88
"CMD_CF_WRITE_CSR3",     // 89
"CMD_CF_READ_REG",       // 90
"CMD_CF_READ_CREG",      // 91
"CMD_CF_READ_DREG",      // 92
"CMD_CF_READ_XCSR",      // 93
"CMD_CF_WRITE_REG",      // 94
"CMD_CF_WRITE_CREG",     // 95
"CMD_CF_WRITE_DREG",     // 96
"CMD_CF_READ_MEM",       // 97
"CMD_CF_WRITE_MEM",      // 98
"CMD_CF_WRITE_XCSR",     // 99

"CMD_READ_STATUS",       // 100
"CMD_WRITE_CONTROL",     // 101
"CMD_WRITE_BKPT",        // 102
"CMD_READ_BKPT",         // 103
"CMD_READ_SPC",          // 104
"CMD_WRITE_SPC",         // 105
"CMD_READ_CCR_PC",       // 106
"CMD_WRITE_CCR_PC",      // 107
"CMD_READ_BLOCK",        // 108
"CMD_WRITE_BLOCK",       // 109
"CMD_MASS_ERASE",        // 110
"CMD_VPPOFF",            // 111
"CMD_RS08_POG",          // 112
"CMD_FLASH_DLSTART",     // 113
"CMD_FLASH_DLEND",       // 114
"TCMD_NEXT_DLDATA",      // 115
NULL,                    // 116
NULL,                    // 117
NULL,                    // 118
NULL,                    // 119
"CMD_CF_SET_DERIVATIVE", // 120
"CMD_CF_READ_REG",       // 121
"CMD_CF_READ_CREG",      // 122
"CMD_CF_READ_DREG",      // 123
"CMD_CF_READ_MEM",       // 124
"CMD_CF_READ_XCSR",      // 125
"CMD_CF_WRITE_REG",      // 126
"CMD_CF_WRITE_CREG",     // 127
"CMD_CF_WRITE_DREG",     // 128
"CMD_CF_WRITE_MEM",      // 129
};

//! \brief Maps a command code to a string
//!
//! @param command Command number
//!
//! @return pointer to static string describing the command
//!
const char *getCommandName(unsigned char command) {
   static char const *commandName = NULL;
   if (command < sizeof(commandTable)/sizeof(commandTable[0]))
      commandName = commandTable[command];
   if (commandName == NULL)
      commandName = "UNKNOWN";
   return commandName;
}

//! Error message string from Error #
static const char *const errorMessages[] = {
    "No Error",                           // 0
    "Illegal parameters",                 // 1
    "General Fail (for compatibility)",   // 2
    "Busy",                               // 3
    "Illegal command",                    // 4
    "No connection to target",            // 5
    "Overrun",                            // 6
    NULL,                                 // 7
    NULL,                                 // 8
    NULL,                                 // 9
    NULL,                                 // 10
    NULL,                                 // 11
    NULL,                                 // 12
    NULL,                                 // 13
    NULL,                                 // 14
    "Unknown Target",                     // 15
    "No Tx routine was found",            // 16
    "No Rx routine was found",            // 17
    "BDM Enable failed",                  // 18
    "Reset pin timeout",                  // 19
    "BKGD pin timeout",                   // 20
    "SYNC timeout",                       // 21
    "Unable to determine speed",          // 22
    "Wrong Flash programming mode",       // 23
    "Busy Flash Programming",             // 24
    "Vdd not removed",                    // 25
    "Vdd not present",                    // 26
    "Vdd not controlled by BDM",          // 27
};

//! \brief Maps a Error Code # to a string
//!
//! @param error Error number
//!
//! @return pointer to static string describing the error
//!
const char *getErrorName(unsigned char error) {
   char const *errorName = NULL;

   if (error < sizeof(errorMessages)/sizeof(errorMessages[0]))
      errorName = errorMessages[error];
   if (errorName == NULL)
      errorName = "UNKNOWN";
   return errorName;
}

//! Error message string from Error #
static const char *const debugCommands[] = {
   "ACKN",              // 0
   "SYNC",              // 1
   "Test Port",         // 2
   "USB Disconnect",    // 3
   "Find Stack size",   // 4
   "Vpp Off",           // 5
   "Vpp On",            // 6
   "Flash 12V Off",     // 7
   "Flash 12V On",      // 8
   "Vdd Off",           // 9
   "Vdd 3.3V On",       // 10
   "Vdd 5V On",         // 11
   "Cycle Vdd",         // 12
   "Measure Vdd",       // 13
};

//! \brief Maps a Debug Command # to a string
//!
//! @param cmd Debug command number
//!
//! @return pointer to static string describing the command
//!
const char *getDebugCommandName(unsigned char cmd) {
   char const *cmdName = NULL;
   if (cmd < sizeof(debugCommands)/sizeof(debugCommands[0]))
      cmdName = debugCommands[cmd];
   if (cmdName == NULL)
      cmdName = "UNKNOWN";
   return cmdName;
}

//! Option command message string from Option command #
static const char *const optionCommands[] = {
   "Get Capabilities",                             // 0
   "Set Target Vdd (Off,3.3V,5V)",                 // 1
   "Set Cycle Target Vdd on reset",                // 2
   "Set Automatic re-connect",                     // 3
   "Set Guess speed (HC12 only)",                  // 4
   "Set Leave Target Powered on Exit",             // 5
   "Set Use Alternate BDM Target Clock",           // 6
   "Set Cycle Target Vdd on connection problems",  // 7
   "Set Use BDM RESET signal",                     // 8
   "Set Trim frequency (SYNC value)",              // 9
   "Set Interface (BKGD/RESET) High/Low",          // 10
   };

//! \brief Maps a Option Command # to a string
//!
//! @param cmd Option command number
//!
//! @return pointer to static string describing the command
//!
const char *getOptionCommandName(unsigned char cmd) {
   char const *cmdName = NULL;
   if (cmd < sizeof(optionCommands)/sizeof(optionCommands[0]))
      cmdName = optionCommands[cmd];
   if (cmdName == NULL)
      cmdName = "UNKNOWN";
   return cmdName;
}


