/*
    Turbo BDM Light ColdFire
    Copyright (C) 2006  Daniel Malik

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Change History
   +====================================================================
   |  20 Oct 2008 |
   +====================================================================
    \endverbatim
*/
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "common_usb.h"
#include "log.h"
#include "hwdesc.h"
#include "libusb.h"
#include "CmdTable.h"
#include "Commands.h"
#include "ICP.h"

static struct usb_device *bdm_devs[10]={NULL};	/* array pointing to all Turbo BDM Light ColdFire devices found terminated by NULL pointer */
static usb_dev_handle *bdm_devh=NULL;

/* provides low level USB functions which talk to the hardware */

//! Initialisation of low-level USB interface
//!
void bdm_usb_init(void) {
   print("===========================================\r\n");
   print("bdm_usb_init()\r\n");
	usb_init();				/* init LIBUSB */
}

//! Find all TBLCF devices attached to the computer
//!
void bdm_usb_find_devices(void) {
	struct usb_bus *libusb_bus;
	struct usb_device *libusb_dev;
  	struct usb_version *version;
	unsigned int i=0;

   print("===========================================\r\n");
   print("bdm_usb_find_devices()\r\n");

   for (i=0; i<(sizeof(bdm_devs)/sizeof(struct usb_device*)); i++)
      bdm_devs[i]=NULL;	 // Clear the list of devices

	version = usb_get_version();
	if (version==NULL) {
		/* something quite wrong... */
		return;
	}
	print("===========================================\r\n");
   print("USBLIB DLL version: %i.%i.%i.%i\r\n",version->dll.major,  version->dll.minor,
                                               version->dll.micro,  version->dll.nano);
	if (version->driver.major==-1) {
		/* driver not running! */
		print("USB: Driver not running\r\n");
		return;
	}
	print("USBLIB Driver version: %i.%i.%i.%i\r\n",version->driver.major,  version->driver.minor,
                                                  version->driver.micro,  version->driver.nano);
	usb_find_busses();		/* enumerate all busses */
	usb_find_devices();		/* enumerate all devices */
	i = 0;
	for (libusb_bus = usb_get_busses(); libusb_bus; libusb_bus = libusb_bus->next) {		/* scan through all busses */
    	for (libusb_dev = libusb_bus->devices; libusb_dev; libusb_dev = libusb_dev->next) {	/* scan through all devices */
#ifdef TBLCF   
  			if ((libusb_dev->descriptor.idVendor==TBLCF_VID)&&(libusb_dev->descriptor.idProduct==TBLCF_PID)) {
#else      
         if (((libusb_dev->descriptor.idVendor==TBDML_VID)&&(libusb_dev->descriptor.idProduct==TBDML_PID)) ||
             ((libusb_dev->descriptor.idVendor==OSBDM_VID)&&(libusb_dev->descriptor.idProduct==OSBDM_PID))) {
#endif
				/* found a device */
				bdm_devs[i++]=libusb_dev;	/* add pointer to the device */
				bdm_devs[i]=NULL;			   /* terminate the list again */
				if (i>=(sizeof(bdm_devs)/sizeof(struct usb_device*)))
				   return;	/* too many devices! */
			}
		}
	}
}

#define DT_STRING          3

unsigned int bdm_get_string_descriptor(unsigned char index, void *buf, int size)
{
  memset(buf, 0, size);

  return usb_control_msg(bdm_devh,
                         USB_ENDPOINT_IN|USB_TYPE_STANDARD, // bmRequestType
                         USB_REQ_GET_DESCRIPTOR,            // bRequest
                         (DT_STRING << 8) + index,          // wValue
                         0,                                 // wIndex
                         buf,                               // data
                         size,                              // size
                         TIMEOUT);                          // timeout
}

//! Count OSBDM/TBDML devices
//!
//! @return Number of devices found
//!
unsigned int bdm_usb_cnt(void) {
	unsigned int i=0;

	while ((i<(sizeof(bdm_devs)/sizeof(struct usb_device*)))&&
	       (bdm_devs[i]!=NULL)) i++;	/* count the entries */

   print("===========================================\r\n");
   print("bdm_usb_cnt() ==> %d\r\n", i);

	return(i);
}

//! Open connection to device enumerated by find_tbdml_devices
//!
//! @param device_no Device number to open
//!
//! @return 0 => Success\n
//!         1 => Error
//!
unsigned char bdm_usb_open(unsigned int device_no) {
   int rc;

   print("===========================================\r\n");
	print("bdm_usb_open( %i )\r\n", device_no);
   if ((device_no >= sizeof(bdm_devs)/sizeof(usb_device)) ||
       (bdm_devs[device_no] == NULL)) {
      print("bdm_usb_open() - Illegal device #\r\n");
      return (1);
      }
	bdm_devh = usb_open(bdm_devs[device_no]);
	if (bdm_devh==NULL) {
      print("bdm_usb_open() - Failed to open device\r\n");
	   //return (1);
      }
	rc = usb_set_configuration(bdm_devh,1);
	if (rc) {
      print("bdm_usb_open() - Failed set configuration, rc = %d\r\n", rc);
	   return(1);  /* Only one valid configuration */
	}
	print("USB Configuration set\r\n");
	rc = usb_claim_interface(bdm_devh,0);
	if (rc) {
      print("bdm_usb_open() - Failed to claim interface, rc = %d\r\n", rc);
	   return(1);		/* Only 1 interface */
	}
	print("USB Interface claimed\r\n");
	return (0);
}

//! Closes connection to the currently open device
//!
void bdm_usb_close(void) {
	if (bdm_devh!=NULL) {
		usb_release_interface(bdm_devh,0);			/* release the interface */
	   print("===========================================\r\n");
		print("USB Interface released\r\n");
		usb_close(bdm_devh);						/* close the device */
		print("USB Device closed\r\n");
		bdm_devh=NULL;								/* indicate that no device is open */
	}
}

/*! \brief Sends a message+data to the TBDML/OSBDM device over EP0
 *
 *  No response is expected from device.
 *
 *  Since the EP0 transfer is unidirectional in this case, data returned by the
 *  device must be read in a separate transfer - this includes any status.
 *
 * @param data \n
 *    data[0]    = N, the number of bytes to send (excluding this byte)\n
 *    data[1]    = Command byte \n
 *    data[2..N] = parameter(s) for command \n
 *
 * @return
 *   0 => USB transfer OK \n
 *   1 => USB transfer Failed
 *
*/
unsigned char bdm_usb_send_ep0(unsigned char * data) {
   unsigned char count = data[0];   // Data count is the first byte
   int rc;
   if (bdm_devh==NULL) {
      print("USB EP0 send: device not open\r\n");
      return(1);
   }
   print("============================\r\n");
   print("USB EP0 send (%s, size=%d):\r\n", getCommandName(data[1]), count);
   if (data[1] == CMD_DEBUG)
      print("Debug cmd = %s\r\n", getDebugCommandName(data[2]));
   if (data[1] == CMD_OPTION)
      print("Option cmd = %s\r\n", getOptionCommandName(data[2]));
   printDump(data, count+1);
   rc=usb_control_msg(bdm_devh,
                      0x40,                    // requestType=Vendor USB_TYPE_VENDOR|USB_ENDPOINT_OUT
                      data[1],                 // request
                      data[2]+(data[3]<<8),    // value
                      data[4]+(data[5]<<8),    // index
                      data+6,                  // data bytes
                      (count>5)?(count-5):0,   // size (# of data bytes)
                      TIMEOUT);                // how long to wait for reply
   if (rc<0) {
      print("USB EP0 send: Send failed (USB error = %d)\r\n", rc);
      return(rc);
   }
   return(0);
}
/*! \brief Sends a message+data to the Device in ICP mode
 *
 *  No response is expected from device.
 *
 *  Since the EP0 transfer is unidirectional in this case, data returned by the
 *  device must be read in a separate transfer - this includes any status.
 *
 * @param request
 * @param wValue
 * @param wIndex
 * @param size
 * @param data
 *
 * @return
 *   0 => USB transfer OK \n
 *   1 => USB transfer Failed
 *
*/
unsigned char bdm_usb_raw_send_ep0(unsigned int  request,
                                   unsigned int  wValue,
                                   unsigned int  wIndex,
                                   unsigned int  size,
                                   unsigned char *data) {
   int rc;
   if (bdm_devh==NULL) {
      print("USB EP0 raw send: device not open\r\n");
      return(1);
   }
//   print("============================\r\n");
//   print("USB EP0 raw send req=%2.2X, val=%2.2X, ind=%d, size=%d\r\n",
//         request, wValue, wIndex, size);
   rc=usb_control_msg(bdm_devh,
                      0x40,      // requestType=Vendor USB_TYPE_VENDOR|USB_ENDPOINT_OUT
                      request,   // request
                      wValue,    // value
                      wValue,    // index
                      data,      // data bytes
                      size,      // size (# of data bytes)
                      TIMEOUT);  // how long to wait for reply
   if (rc<0) {
      print("USB EP0 raw send: Send failed (USB error = %d)\r\n", rc);
      return(rc);
   }
   return(0);
}

/*! \brief Sends a message+data to the Device in ICP mode
 *
 *  No response is expected from device.
 *
 *  Since the EP0 transfer is unidirectional in this case, data returned by the
 *  device must be read in a separate transfer - this includes any status.
 *
 * @param request
 * @param wValue
 * @param wIndex
 * @param size
 * @param data
 *
 * @return
 *   0 => USB transfer OK \n
 *   1 => USB transfer Failed
 *
*/
unsigned char bdm_usb_raw_recv_ep0(unsigned int  request,
                                   unsigned int  wValue,
                                   unsigned int  wIndex,
                                   unsigned int  size,
                                   unsigned char *data) {
   int rc;
   if (bdm_devh==NULL) {
      print("USB EP0 raw receive: device not open\r\n");
      return(1);
   }
//   print("============================\r\n");
//   print("USB EP0 raw receive req=%2.2X, val=%2.2X, ind=%d, size=%d\r\n",
//         request, wValue, wIndex, size);
   rc=usb_control_msg(bdm_devh,
                      0xC0,      // requestType=Vendor USB_TYPE_VENDOR|USB_ENDPOINT_OUT
                      request,   // request
                      wValue,    // value
                      wValue,    // index
                      data,      // data bytes
                      size,      // size (# of data bytes)
                      TIMEOUT);  // how long to wait for reply
   if (rc<0) {
      print("USB EP0 raw receive: Send failed (USB error = %d)\r\n", rc);
      return(rc);
   }
   return(0);
}

/*! \brief Sends a message of up to 5 bytes to the TBDML/OSBDM device over EP0.
 *
 *  An immediate response is expected
 *
 * @param data - Entry \n
 *    data[0]    = N, the number of bytes to receive from the device \n
 *    data[1]    = Command byte \n
 *    data[2..5] = parameter(s) for OSBDM command \n
 *    data[6]    = N-1 = data transfer size - USB Handler on BDM module sets this byte up\n
 *    data[7]    = 0
 *
 * @param data - Exit \n
 *    data[0]      = cmd response from OSBDM\n
 *    data[1..N-1] = data response from the device (cleared on error)\n
 *
 * @return  0 => USB transfer OK, valid response from device \n
 *          1 => USB transfer Failed, or unexpected response from device \n
 *               data[1..N] cleared
 */
unsigned char bdm_usb_recv_ep0(unsigned char * data) {
   unsigned char size = data[0];   // transfer size is the first byte
   unsigned char cmd  = data[1];   // OSBDM/TBDML Command byte
 	int rc;
	if (bdm_devh==NULL) {
		print("USB EP0 receive request: device not open\r\n");
		return(1);
	}

	//size++; // +1 for possible debug byte
   print("============================\r\n");
	print("USB EP0 receive request (%s, size=%d):\r\n", getCommandName(cmd), size);
   if (data[1] == CMD_DEBUG)
      print("Debug cmd = %s\r\n", getDebugCommandName(data[2]));
   if (data[1] == CMD_OPTION)
      print("Option cmd = %s\r\n", getOptionCommandName(data[2]));
	printDump(data,6);
   rc=usb_control_msg(bdm_devh,
                      0xC0,                   // bmRequestType - user USB_TYPE_VENDOR|USB_ENDPOINT_IN
                      cmd,                    // bRequest
                      data[2]+(data[3]<<8),   // wValue
                      data[4]+(data[5]<<8),   // wIndex
                      data,                   // ptr to data buffer (for Rx)
                      size,                   // wLength = size of transfer
                      TIMEOUT                 // timeout
                      );
   if (rc<0) {
      print("USB EP0 receive: Transfer failed (USB error = %d)\r\n", rc);
      return(1);
   }
   if ((data[0] != BDM_RC_OK) && (data[0] != cmd)) { // Error?
      print("USB EP0 receive: Cmd Failed (%s):\r\n", getErrorName(data[0]));
      print("USB EP0 receive (size = %d, recvd = %d):\r\n", size, rc);
      printDump(data,rc);
      memset(&data[1], 0x00, size-1);
      return 1;
   }
   data[0] = 0;   // So dump contains nice RC
   print("USB EP0 receive (size = %d, recvd = %d):\r\n", size, rc);
   printDump(data,rc);
   data[0] = cmd; // For compatibility
   return(0);
}

#if 0
/* sends a message to the TBDML device over EP2 */
/* returns 0 on success and 1 on error */
/* data the device wants to return need to be read out in separate recv transaction */
unsigned char bdm_usb_send_ep2(unsigned char *data) {
	unsigned char * count = data;		/* data count is the first byte of the message */
 	int i;

	if (bdm_devh==NULL) {
		print("USB EP2 send: device not open\r\n");
		return(1);
	}
	print("USB EP2 send:\r\n");
	printDump(data,(*count)+1);
	i=usb_bulk_write(bdm_devh, 0x02, data, (*count)+1, TIMEOUT);
	if (i<0) return(1); else return(0);
}

/* receives data from EP2 */
/* returns 0 on success and 1 on error */
/* data count in the message is number of bytes EXPECTED/REQUIRED from the device */
unsigned char bdm_usb_recv_ep2(unsigned char * data) {
	unsigned char count = *data;		/* data count is the first byte of the message */
 	int i;
	if (bdm_devh==NULL) {
		print("USB EP2 receive request: device not open\r\n");
		return(1);
	}
	print("USB EP2 receive request:\r\n");
	printDump(data,6);
	i=usb_bulk_read(bdm_devh, 0x82, data, count, TIMEOUT);
	print("USB EP2 receive (%d byte(s)):\r\n",i);
	printDump(data,count);
	if (i<0) return(1); else return(0);
}
#endif
